
import pathlib


from generallibrary import VerInfo, initBases, TreeDiagram

from generalfile.errors import InvalidCharacterError
from generalfile.path_lock import Path_ContextManager
from generalfile.path_operations import Path_Operations
from generalfile.path_strings import Path_Strings
from generalfile.optional_dependencies.path_spreadsheet import Path_Spreadsheet
from generalfile.optional_dependencies.path_text import Path_Text
from generalfile.optional_dependencies.path_cfg import Path_Cfg


@initBases
class Path(TreeDiagram, Path_ContextManager, Path_Operations, Path_Strings, Path_Spreadsheet, Path_Text, Path_Cfg):
    """ Immutable cross-platform Path.
        Built on pathlib and TreeDiagram.
        Implements rules to ensure cross-platform compatability.
        Adds useful methods. """
    verInfo = VerInfo()
    path_delimiter = verInfo.pathDelimiter
    Path = ...

    def __init__(self, path=None, parent=None):
        path = self._scrub(str_path="" if path is None else str(path))
        self.path = self.data_keys_add(key="path", value=path, unique=True)

        self._path = pathlib.Path(self.path)

    def get_parent(self, index=0):
        """ Override to generate all parents if direct parent is None. """
        if self._parent is None:
            self._generate_parents()
        return TreeDiagram.get_parent(self=self, index=index)

    def _generate_parents(self):
        path = None
        for pathlib_path in reversed(self._path.parents):
            path = self.Path(path="" if str(pathlib_path) == "." else str(pathlib_path), parent=path)
        self.set_parent(path)

    def __str__(self):
        return self.path

    def __repr__(self):
        return self.__str__()

    def __truediv__(self, other):
        """ :rtype: generalfile.Path """
        return self.Path(self._path / str(other))
    
    def __eq__(self, other):
        return str(self) == str(other)

    def __hash__(self):
        return hash(str(self))

    def __contains__(self, item):
        return self.path.__contains__(item)

    def _scrub(self, str_path):
        str_path = self._replace_delimiters(str_path=str_path)
        str_path = self._invalid_characters(str_path=str_path)
        str_path = self._trim(str_path=str_path)
        return str_path

    def _replace_delimiters(self, str_path):
        str_path = str_path.replace("/", self.path_delimiter)
        str_path = str_path.replace("\\", self.path_delimiter)
        # str_path = str_path.replace(self.path_delimiter_alternative, self.path_delimiter)  # Don't remember why I commented this
        return str_path

    def _invalid_characters(self, str_path):
        # Simple invalid characters testing from Windows
        for character in '<>"|?*':
            if character in str_path:
                raise InvalidCharacterError(f"Invalid character '{character}' in '{str_path}'")
        if ":" in str_path:
            if not self.verInfo.pathRootHasColon:
                raise InvalidCharacterError(f"Path has a colon but '{self.verInfo.os}' doesn't use colon for path root: '{str_path}'")
            if str_path[1] != ":":
                raise InvalidCharacterError(f"Path has a colon but there's no colon at index 1: '{str_path}'")
            if len(str_path) >= 3 and str_path[2] != self.path_delimiter:
                raise InvalidCharacterError(f"Path has a colon but index 2 is not a delimiter: '{str_path}'")
            if ":" in str_path[2:]:
                raise InvalidCharacterError(f"Path has a colon that's not at index 1: '{str_path}'")
        if str_path.endswith("."):
            raise InvalidCharacterError(f"Path cannot end with a dot ('.').")
        return str_path

    def _trim(self, str_path):
        if not self.verInfo.pathRootIsDelimiter and str_path.startswith(self.path_delimiter):
            str_path = str_path[1:]
        if str_path.endswith(self.path_delimiter):
            str_path = str_path[0:-1]
        return str_path

    def view(self, only_last_part=True, indent=1, relative=False, custom_repr=None, print_out=True):
        """ Override view to use default custom repr. """
        if only_last_part and custom_repr is None:
            custom_repr = lambda path: path.parts()[-1]
        return TreeDiagram.view(self=self, indent=indent, relative=relative, custom_repr=custom_repr, print_out=print_out)

setattr(Path, "Path", Path)



















































from generallibrary import deco_cache, initBases

import re


class Path_Text:
    """ Text methods for Path. """
    @property
    @deco_cache()
    def text(self):
        """ Easy access to a dynamically one-time created Text class, initalized with self (Path). """
        return self._text()(self)

    @staticmethod
    @deco_cache()
    def _text():
        @initBases
        class _Text(_Extension):
            def write(self, text=None, overwrite=False):
                """ Write to this path with a given string. """
                with self.WriteContext(self.path, overwrite=overwrite) as write_path:
                    with open(str(write_path), "w") as file:
                        file.write(text)

            def read(self):
                """ Read from this path to get a string. """
                with self.ReadContext(self.path) as read_path:
                    with open(str(read_path), "r") as file:
                        return file.read()

            def append(self, string, newline=False):
                """ Append to this path with a given string. """
                with self.AppendContext(self.path) as append_path:
                    with open(str(append_path), "a") as file:
                        if newline:
                            file.write("\n")
                        file.write(string)

            def replace(self, d, regex=False):
                """ Replace matches in this path with a given dictionary. """
                with self.path.lock():
                    text = self.path.text.read()
                    for key, value in d.items():
                        if regex:
                            text = re.sub(key, value, text)
                        else:
                            text = text.replace(key, value)
                    self.path.text.write(text, overwrite=True)



        return _Text


from generalfile.path_operations import _Extension










































import unittest

from generalfile import Path
from generalfile.test.setup_workdir import setup_workdir
from generalfile.errors import InvalidCharacterError


class PathTest(unittest.TestCase):
    def setUp(self):
        """Set working dir and clear folder. Set path delimiter to '/' for testing."""
        Path.path_delimiter = "/"
        setup_workdir()


class FileTest(PathTest):
    def test_path(self):
        self.assertRaises(InvalidCharacterError, Path, "hello:there")
        self.assertRaises(InvalidCharacterError, Path, "hello<")
        self.assertRaises(InvalidCharacterError, Path, "hello>")
        self.assertRaises(InvalidCharacterError, Path, "hello.")

    def test_addPath(self):
        self.assertEqual(Path("foo/bar"), Path("foo") / "bar")
        self.assertEqual(Path("foo/bar"), Path("foo") / Path("bar"))
        self.assertEqual(Path("foo.txt/folder"), Path("foo.txt") / "folder")
        self.assertEqual(Path("folder/foo.txt"), Path("folder") / "foo.txt")

    def test_parts(self):
        path = Path("folder/folder2/test.txt")
        self.assertEqual(["folder", "folder2", "test.txt"], path.parts())

        self.assertEqual("foo", Path("foo/bar").parts()[0])
        self.assertEqual("bar", Path("foo/bar").parts()[1])

        self.assertEqual(not Path.verInfo.pathRootIsDelimiter, bool(Path().absolute().parts()[0]))

    def test_name(self):
        path = Path("folder/test.txt")
        self.assertEqual("test.txt", path.name())

        self.assertEqual("folder/foobar.txt", path.with_name("foobar.txt"))
        self.assertEqual("folder/hi", path.with_name("hi"))

    def test_stem(self):
        path = Path("folder/test.txt")
        self.assertEqual("test", path.stem())
        self.assertEqual("folder/foobar.txt", path.with_stem("foobar"))

        path = Path("folder/test.foo.txt.bar")
        self.assertEqual("test.foo.txt", path.stem())
        self.assertEqual("folder/foo.bar", path.with_stem("foo"))

    def test_true_stem(self):
        path = Path("folder/test.txt")
        self.assertEqual("test", path.true_stem())
        self.assertEqual("folder/foobar.txt", path.with_true_stem("foobar"))

        path = Path("folder/test.foo.txt.bar")
        self.assertEqual("test", path.true_stem())
        self.assertEqual("folder/yo.foo.txt.bar", path.with_true_stem("yo"))

    def test_suffixes(self):
        path = Path("folder/test.hello.txt")
        self.assertEqual([".hello", ".txt"], path.suffixes())
        self.assertEqual("folder/test.tsv", path.with_suffixes([".tsv"]))

    def test_suffix(self):
        path = Path("folder/test.txt")
        self.assertEqual(".txt", path.suffix())

        path = path.with_suffix(".tsv")
        self.assertEqual("folder/test.tsv", path)

        path = path.with_suffix(".csv")
        self.assertEqual("folder/test.csv", path)

        path = path.with_suffix(".BACKUP", -2)
        self.assertEqual("folder/test.BACKUP.csv", path)

        path = path.with_suffix(".test", -2)
        self.assertEqual("folder/test.test.csv", path)

        path = path.with_suffix(None, 0)
        self.assertEqual("folder/test.csv", path)

        path = path.with_suffix(".foo", 2)
        self.assertEqual("folder/test.csv.foo", path)

        path = path.with_suffix(".bar", 3)
        self.assertEqual("folder/test.csv.foo.bar", path)

        path = path.with_suffix(".clamped", 5)
        self.assertEqual("folder/test.csv.foo.bar.clamped", path)

        path = path.with_suffix(".clamped", -10)
        self.assertEqual("folder/test.clamped.csv.foo.bar.clamped", path)

        path = path.with_suffix(None, 10)
        self.assertEqual("folder/test.clamped.csv.foo.bar", path)

        path = path.with_suffix(None, -10)
        self.assertEqual("folder/test.csv.foo.bar", path)

    def test_parent(self):
        path = Path("folder/foobar/test.txt")
        self.assertEqual(Path("folder/foobar"), path.get_parent())
        self.assertEqual(Path("folder/foobar"), path.get_parent(0))
        self.assertEqual(Path("folder"), path.get_parent(1))
        self.assertEqual(Path(), path.get_parent(2))
        self.assertEqual(None, path.get_parent(3))
        self.assertEqual(None, path.get_parent(99))
        self.assertEqual(None, path.get_parent(-99))

    def test_startswith(self):
        self.assertFalse(Path("file.txt").startswith("folder"))
        self.assertTrue(Path("file.txt").startswith("file"))
        self.assertFalse(Path("folder/file.txt").startswith("file.txt"))
        self.assertFalse(Path("folder/file.txt").absolute().startswith("folder"))

        self.assertTrue(Path("folder/file.txt").startswith("folder"))
        self.assertTrue(Path("file.txt").startswith("file.txt"))
        self.assertTrue(Path("file.SUFFIX.txt").startswith("file.SUFFIX.txt"))
        self.assertFalse(Path("filE.txt").startswith("file.txt"))

    def test_endswith(self):
        self.assertFalse(Path("file.txt").endswith("folder"))
        self.assertFalse(Path("file.txt").endswith("file"))
        self.assertFalse(Path("folder/file.txt").endswith("folder"))
        self.assertFalse(Path("folder/file.txt").absolute().endswith("file"))

        self.assertTrue(Path("folder/file.txt").endswith("file.txt"))
        self.assertTrue(Path("folder/file.txt").endswith("txt"))
        self.assertTrue(Path("file.txt").endswith("file.txt"))
        self.assertFalse(Path("filE.txt").endswith("file.txt"))

    def test_remove_start(self):
        self.assertEqual(Path(), Path("test.txt").remove_start("test.txt"))
        self.assertEqual(Path("folder/test.txt"), Path("folder/test.txt").remove_start("Folder"))
        self.assertEqual(Path("test.txt"), Path("folder/test.txt").remove_start("folder"))
        self.assertEqual(Path("folder/test.txt"), Path("folder/test.txt").remove_start("test"))

        if Path.verInfo.pathRootIsDelimiter:
            self.assertEqual(Path("test.txt"), Path("folder/test.txt").remove_start("folder"))

    def test_remove_end(self):
        self.assertEqual(Path(), Path("test.txt").remove_end("test.txt"))
        self.assertEqual(Path("test"), Path("test.txt").remove_end(".txt"), "test")
        self.assertEqual(Path("folder"), Path("folder/test.txt").remove_end("test.txt"))
        self.assertEqual(Path("folder/test.txt"), Path("folder/test.txt").remove_end("test"))

    def test_absolute(self):
        path = Path("test.txt")
        self.assertEqual(False, path.is_absolute())
        self.assertEqual(True, path.is_relative())

        path = path.absolute()
        self.assertEqual(True, path.is_absolute())
        self.assertEqual(False, path.is_relative())

        path = path.relative()
        self.assertEqual(False, path.is_absolute())
        self.assertEqual(True, path.is_relative())


        path = Path("folder/folder2/file.txt")
        self.assertEqual(Path("folder2/file.txt"), path.relative("folder"))
        self.assertEqual(path.relative("folder"), "folder2/file.txt")
        self.assertEqual(path.relative("folder/folder2"), "file.txt")



    def test_is_file_or_folder(self):
        Path("folder.txt/file.txt").write()
        self.assertEqual(True, Path("folder.txt").is_folder())
        self.assertEqual(False, Path("folder.txt").is_file())

        self.assertEqual(True, Path("folder.txt/file.txt").is_file())
        self.assertEqual(False, Path("folder.txt/file.txt").is_folder())

    def test_exists(self):
        path = Path("folder/test.txt")
        self.assertEqual(False, path.exists())
        self.assertEqual(False, Path("folder").exists())

        path.write()
        self.assertEqual(True, path.exists())
        self.assertEqual(True, Path("folder").exists())
        self.assertEqual(False, Path("folder/test").exists())

        Path("folder").delete()
        self.assertEqual(False, path.exists())
        self.assertEqual(False, Path("folder").exists())


    def test_working_dir(self):
        self.assertEqual(True, Path.get_working_dir().is_absolute())
        self.assertEqual(Path().absolute(), Path.get_working_dir())

        Path("folder").set_working_dir()
        self.assertEqual(True, Path.get_working_dir().endswith("folder"))
        self.assertEqual(Path().absolute(), Path.get_working_dir())

    def test_same_destination(self):
        path = Path("folder")
        self.assertEqual(True, path.same_destination(Path() / "folder"))
        self.assertEqual(True, path.same_destination(path.absolute()))
        self.assertEqual(True, path.same_destination("folder"))

    def test_write(self):
        self.assertEqual('"foobar"', Path("test.txt").write("foobar"))
        self.assertEqual("foobar", Path("test.txt").read())

        self.assertEqual('"foobar"', Path("test2").write("foobar"))
        self.assertEqual("foobar", Path("test2").read())

        self.assertEqual('"foobar"', Path("test2.doesntexist").write("foobar"))
        self.assertEqual("foobar", Path("test2.doesntexist").read())

        self.assertEqual('"foobar"', Path("folder/test.txt").write("foobar"))
        self.assertEqual("foobar", Path("folder/test.txt").read())

    def test_rename(self):
        Path("folder/test.txt").write()

        Path("folder/test.txt").rename(name="hello.txt")
        self.assertTrue(Path("folder/hello.txt").exists())
        self.assertFalse(Path("folder/test.txt").exists())

        Path("folder").rename(name="folder2")
        self.assertTrue(Path("folder2").exists())
        self.assertFalse(Path("folder").exists())

        Path("folder2/hello.txt").rename(name="foo.txt")
        self.assertTrue(Path("folder2/foo.txt").exists())

        Path("folder2/foo.txt").rename(name="foo.TEST.txt")
        self.assertTrue(Path("folder2/foo.TEST.txt").exists())

        Path("folder2/foo.TEST.txt").rename(name="foobar")
        self.assertTrue(Path("folder2/foobar").is_file())

        Path("folder2/foobar").rename(suffix=".test")
        self.assertTrue(Path("folder2/foobar.test").exists())

        Path("folder2/foobar.test").rename(stem="hello")
        self.assertTrue(Path("folder2/hello.test").exists())

    def test_copy(self):
        Path("folder/test.txt").write()
        Path("folder/test.txt").copy("foo.txt")
        self.assertEqual(True, Path("folder/foo.txt").exists())

        Path("folder").copy("new")
        self.assertEqual(True, Path("new/foo.txt").exists())

        Path("new/foo.txt").copy("new/bar.txt")
        self.assertEqual(True, Path("new/bar.txt").exists())

    def test_copy_to_folder(self):
        Path("folder/test.txt").write()
        Path("folder/test2.txt").write()

        Path("folder").copy_to_folder("folder2")
        self.assertEqual(True, Path("folder2/test.txt").exists())
        self.assertEqual(True, Path("folder2/test2.txt").exists())
        self.assertEqual(True, Path("folder/test2.txt").exists())

        Path("folder/test.txt").copy_to_folder("")
        self.assertEqual(True, Path("test.txt").exists())
        self.assertEqual(False, Path("test2.txt").exists())

        Path("folder").copy_to_folder(Path(), overwrite=True)
        self.assertEqual(True, Path("test2.txt").exists())

    def test_move(self):
        Path("folder/test.txt").write(5)
        Path("folder/test2.txt").write()

        Path("folder").move("folder2")
        self.assertEqual(False, Path("folder").exists())
        self.assertEqual(True, Path("folder2/test.txt").exists())
        self.assertEqual(True, Path("folder2/test2.txt").exists())

        Path("folder2/test.txt").move("")
        self.assertEqual(True, Path("test.txt").exists())
        self.assertEqual(False, Path("test2.txt").exists())
        self.assertEqual(False, Path("folder2/test.txt").exists())

        Path("folder/test.txt").write(2)
        with self.assertRaises(FileExistsError):
            Path("folder").move(Path())

        self.assertEqual(5, Path("test.txt").read())
        Path("folder").move(Path(), overwrite=True)
        self.assertEqual(2, Path("test.txt").read())

    def test_create_folder(self):
        path = Path("folder/folder2.txt")
        path.create_folder()

        self.assertEqual(True, path.is_folder())

    def test_trash_and_delete(self):
        for method in ("trash", "delete"):
            path = Path("file.txt")
            self.assertEqual(False, path.exists())
            self.assertEqual(False, getattr(path, method)())

            path.write()
            self.assertEqual(True, path.exists())
            self.assertEqual(True, getattr(path, method)())
            self.assertEqual(False, getattr(path, method)())

            path = Path("folder/file.txt")
            self.assertEqual(False, path.exists())
            self.assertEqual(False, getattr(path, method)())

            path.write()
            self.assertEqual(True, path.exists())
            self.assertEqual(True, getattr(path.get_parent(), method)())
            self.assertEqual(False, getattr(path.get_parent(), method)())
            self.assertEqual(False, Path("folder").exists())

    def test_trash_and_delete_folder_content(self):
        for method in ("trash_folder_content", "delete_folder_content"):
            setup_workdir()

            mainPath = Path("folder")
            path = mainPath / "file.txt"
            path2 = mainPath / "folder2/file2.txt"
            self.assertEqual(False, mainPath.exists())
            self.assertEqual(False, getattr(mainPath, method)())

            for targetPath in (mainPath, ):
                path.write()
                path2.write()
                self.assertEqual(True, getattr(targetPath, method)())
                self.assertEqual(False, getattr(targetPath, method)())
                self.assertEqual(True, mainPath.exists())
                self.assertEqual(False, path.exists())
                self.assertEqual(False, path2.exists())

    def test_get_paths(self):
        Path("test.txt").write()
        Path("folder/test2.txt").write()
        Path("folder/test3.txt").write()

        self.assertEqual(1, len(list(Path().get_paths_recursive(depth=1))))
        self.assertEqual(2, len(list(Path().get_paths_recursive(depth=1, include_self=True))))

        self.assertEqual(0, len(list(Path("test.txt").get_paths_recursive(depth=1))))
        self.assertEqual(1, len(list(Path("test.txt").get_paths_recursive(depth=1, include_self=True))))
        self.assertEqual(1, len(list(Path("test.txt").get_paths_recursive(depth=1, include_folders=True))))

        self.assertEqual(3, len(list(Path().get_paths_recursive(depth=3))))
        self.assertEqual(4, len(list(Path().get_paths_recursive(depth=3, include_self=True))))
        self.assertEqual(4, len(list(Path().get_paths_recursive(depth=-1, include_self=True))))
        self.assertEqual(2, len(list(Path().get_paths_recursive(depth=1, include_self=True))))

        self.assertEqual(1, len(list(Path("folder/test2.txt").get_paths_recursive())))

    def test_time_created_and_modified(self):
        path = Path("test.txt")
        methods = (path.seconds_since_creation, path.seconds_since_modified)

        for method in methods:
            self.assertRaises(AttributeError, method)

        path.write()

        for method in methods:
            self.assertGreater(method(), 0)

        # Think you need to flush and stuff to make this work for windows atleast
        # self.assertEqual(methods[0](), methods[1]())
        # path.write("foobar", overwrite=True)
        # self.assertNotEqual(methods[0](), methods[1]())

    def test_getitem(self):
        self.assertEqual("f", Path("foobar")[0])
        self.assertEqual("fo", Path("foobar")[0:2])
        self.assertEqual("raboof", Path("foobar")[-1::-1])

    def test_iter(self):
        self.assertEqual(["f", "o", "o"], list(Path("foo")))
        self.assertIn("foo", Path("foobar"))

    def test_threads(self):
        from generallibrary import sleep
        import multiprocessing as mp
        threads = []
        queue = mp.Queue()
        count = 2
        for i in range(count):
            threads.append(mp.Process(target=threadTest, args=(queue, i)))
        for thread in threads:
            thread.start()

        results = []
        for i in range(count):
            get = queue.get()
            self.assertNotIn(get, results)
            results.append(get)

        self.assertEqual(len(results), count)

def threadTest(queue, i):
    queue.put(int(Path("test.txt").write(i, overwrite=True)))

if __name__ == "__main__":
    x = unittest.main()
















































from generallibrary import deco_cache, initBases
import configparser
import json



class Path_Cfg:
    """ Cfg methods for Path. """
    @property
    @deco_cache()
    def cfg(self):
        """ Easily read and write to cfg files with a dictionary.
            Easy access to a dynamically one-time created Cfg class, initalized with self (Path). """
        return self._cfg()(self)

    @staticmethod
    @deco_cache()
    def _cfg():
        @initBases
        class _Cfg(_Extension):
            def write(self, dict_=None, overwrite=False):
                """ Write to this path with a given dictionary. """
                with self.WriteContext(self.path, overwrite=overwrite) as write_path:
                    with open(str(write_path), "w") as file:
                        config = configparser.RawConfigParser()
                        config.read_dict(dictionary=dict_)
                        config.write(file)

            def _read_json_cast(self, value):
                try:
                    return json.loads(value.replace("'", '"'))
                except json.decoder.JSONDecodeError:
                    return value

            def read(self):
                """ Read from this path to get a dictionary. """
                with self.ReadContext(self.path) as read_path:
                    config = configparser.RawConfigParser()
                    config.read(str(read_path))
                    return {s: {key: self._read_json_cast(value) for key, value in config.items(s)} for s in config.sections()}

            def append(self, dict_):
                """ Update this cfg with a dictionary. """
                with self.AppendContext(self.path) as append_path:
                    read = append_path.cfg.read()
                    read.update(dict_)
                    append_path.cfg.write(dict_=dict_, overwrite=True)

        return _Cfg


from generalfile.path_operations import _Extension









































"""Just contains class for File handling tests"""


def setup_workdir():
    """ Class to set up working dir for tests, File extensions import this class.
        Set working dir and clear it after it's made sure it's correct path."""
    path = Path(__file__).get_parent() / "tests"
    path.set_working_dir()

    if not path.get_working_dir().endswith("test/tests"):
        raise EnvironmentError(f"Failed setting correct working dir, should be ..test/tests but it's {path}")

    path.delete_folder_content()


from generalfile.path import Path




from setuptools import setup, find_namespace_packages
from os import path
import configparser
import json
from itertools import chain



def remove_duplicates(l):
    """ Remove all duplicates in a list.
        Values must be hashable as they are passed through as dict keys. (Lists work but not Dicts) """
    return list(set(l))

class Cfg:
    """ Convert cfg file to json. """
    def __init__(self, path):
        self.config = configparser.RawConfigParser()
        self.config.read(path)

    def __call__(self, section, option):
        result = self.config.get(section, option)
        try:
            return json.loads(self.config.get(section, option))
        except json.decoder.JSONDecodeError:
            return result

cfg = Cfg("package_specific.cfg")



this_directory = path.abspath(path.dirname(__file__))  # Same here, could probably use generalfile itself
with open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

extras_require = cfg("setup", "extras_require")
extras_require["full"] = remove_duplicates([package for package in chain(*list(extras_require.values()))])

classifiers=[
    "Operating System :: Microsoft :: Windows :: Windows 7",
    "Operating System :: Microsoft :: Windows :: Windows 10",
    "Operating System :: POSIX :: Linux",
    "Operating System :: MacOS",
    "Programming Language :: Python :: 3.8",
    "Programming Language :: Python :: 3.9",
    "License :: OSI Approved :: MIT License",
]
classifiers.extend(cfg("setup", "classifiers"))

install_requires = cfg("setup", "install_requires")

setup(
    author='Rickard "Mandera" Abraham',
    long_description_content_type="text/markdown",
    url=f"https://github.com/ManderaGeneral/{ cfg('setup', 'name') }",
    license="MIT",
    python_requires=">= 3.8, < 3.10",
    packages=find_namespace_packages(),

    name=cfg("setup", "name"),
    version=cfg("setup", "version"),
    description=cfg("setup", "description"),

    install_requires=install_requires,
    extras_require=extras_require,
    long_description=long_description,
    classifiers=classifiers,
)
